unit SetupDi;

interface

uses
	WinAPI.Windows;

const
	DIGCF_DEFAULT         = $00000001;  // only valid with DIGCF_DEVICEINTERFACE
	DIGCF_PRESENT         = $00000002;
	DIGCF_ALLCLASSES      = $00000004;
	DIGCF_PROFILE         = $00000008;
	DIGCF_DEVICEINTERFACE = $00000010;

	USB_DEVICE_DESCRIPTOR_TYPE = 1;

type
	SP_DEVICE_INTERFACE_DATA = record
		cbSize: DWORD;
		InterfaceClassGuid: TGUID;
		Flags: DWORD;
		Reserved: Pointer;
	end;
	PSP_DEVICE_INTERFACE_DATA = ^SP_DEVICE_INTERFACE_DATA;

	SP_DEVINFO_DATA = record
		cbSize: LongWord;
		ClassGUID: TGUID;
		DevInst: LongWord;
		Reserved: Pointer;
	end;
	PSP_DEVINFO_DATA = ^SP_DEVINFO_DATA;

	SP_DEVICE_INTERFACE_DETAIL_DATA_A = record
		cbSize: LongWord;
		DevicePath: array[0..0] of AnsiChar;
	end;
	SP_DEVICE_INTERFACE_DETAIL_DATA_W = record
		cbSize: LongWord;
		DevicePath: array[0..0] of WideChar;
	end;
	PSP_DEVICE_INTERFACE_DETAIL_DATA_A = ^SP_DEVICE_INTERFACE_DETAIL_DATA_A;
	PSP_DEVICE_INTERFACE_DETAIL_DATA_W = ^SP_DEVICE_INTERFACE_DETAIL_DATA_W;

	USB_DEV_DSC = packed record
		bLength     : byte; bDscType  : byte;  bcdUSB      : word;
		bDevCls     : byte; bDevSubCls: byte;  bDevProtocol: byte;
		bMaxPktSize0: byte; idVendor  : word;  idProduct   : word;
		bcdDevice   : word; iMFR      : byte;  iProduct    : byte;
		iSerialNum  : byte; bNumCfg   : byte;
	end;

function SetupDiEnumDeviceInterfaces(DeviceInfoSet: THandle; DeviceInfoData: PSP_DEVINFO_DATA; const InterfaceClassGuid: TGUID; MemberIndex: LongWord; var DeviceInterfaceData: SP_DEVICE_INTERFACE_DATA): BOOL; stdcall; external 'SetupAPI.dll';
function SetupDiDestroyDeviceInfoList(DeviceInfoSet: THandle): BOOL; stdcall; external 'SetupAPI.dll';
function SetupDiGetClassDevsA(ClassGuid: PGUID; Enumerator: PAnsiChar; hwndParent: THandle; Flags: LongWord): THandle; stdcall; external 'SetupAPI.dll';
function SetupDiGetClassDevsW(ClassGuid: PGUID; Enumerator: PWideChar; hwndParent: THandle; Flags: LongWord): THandle; stdcall; external 'SetupAPI.dll';
function SetupDiGetDeviceInterfaceDetailA(DeviceInfoSet: THandle; DeviceInterfaceData: PSP_DEVICE_INTERFACE_DATA; DeviceInterfaceDetailData: PSP_DEVICE_INTERFACE_DETAIL_DATA_A; DeviceInterfaceDetailDataSize: LongWord; pRequiredSize: PLongWord; DeviceInfoData: PSP_DEVINFO_DATA): BOOL; stdcall; external 'SetupAPI.dll';
function SetupDiGetDeviceInterfaceDetailW(DeviceInfoSet: THandle; DeviceInterfaceData: PSP_DEVICE_INTERFACE_DATA; DeviceInterfaceDetailData: PSP_DEVICE_INTERFACE_DETAIL_DATA_W; DeviceInterfaceDetailDataSize: LongWord; pRequiredSize: PLongWord; DeviceInfoData: PSP_DEVINFO_DATA): BOOL; stdcall; external 'SetupAPI.dll';

type
	TStringArray = array of WideString;

function GetDevicePaths(const ClassGUID: TGUID): TStringArray;



implementation

function GetDevicePaths(const ClassGUID: TGUID): TStringArray;
var
	DevInterfaceData: SP_DEVICE_INTERFACE_DATA;
	MemberIndex: Integer;
	hDevInfo: THandle;

	L: LongWord;
	pDetailData: PSP_DEVICE_INTERFACE_DETAIL_DATA_W;

	Buf: WideString;
	I, RL: Integer;

begin
	RL := 0;
	SetLength(Result, RL);

	hDevInfo := SetupDiGetClassDevsW(@ClassGUID, nil, 0, DIGCF_PRESENT or DIGCF_DEVICEINTERFACE);
	if hDevInfo = 0 then Exit;

	DevInterfaceData.cbSize := sizeof(DevInterfaceData);
	MemberIndex := 0;
	while SetupDiEnumDeviceInterfaces(hDevInfo, nil, ClassGUID, MemberIndex, DevInterfaceData) do begin
		L := 0;
		SetupDiGetDeviceInterfaceDetailW(hDevInfo, @DevInterfaceData, nil, 0, @L, nil);

		GetMem(pDetailData, L);
		if SizeOf(Pointer) = 8 then
			pDetailData.cbSize := 8 //The fixed portion is 4. Dunno where 8 comes from.
		else
			pDetailData.cbSize := 6 //The fixed portion is 4. Dunno where 6 comes from.
		;

		if not SetupDiGetDeviceInterfaceDetailW(hDevInfo, @DevInterfaceData, pDetailData, L, nil, nil) then begin
			FreeMem(pDetailData);
			Inc(MemberIndex);
			Continue;
		end;

		SetString(Buf, PWideChar(@pDetailData.DevicePath[0]), L - 4);
		FreeMem(pDetailData);
		I := Pos(AnsiChar(#0), Buf);
		if I <> 0 then SetLength(Buf, I - 1);

		SetLength(Result, RL + 1);
		Result[RL] := Buf;
		Inc(RL);
		Buf := '';

		Inc(MemberIndex);
	end;
	SetupDiDestroyDeviceInfoList(hDevInfo);
end;

end.
